@@ -26,8 +26,29 @@
 /* The maximum length of PIN */
 #define MAX_PIN_LENGTH   256
 
-#ifndef OPENSSL_NO_EC
+#if OPENSSL_VERSION_NUMBER < 0x40000000L
 # if OPENSSL_VERSION_NUMBER >= 0x30000000L
+static int pkey_ex_index = 0;
+# endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L*/
+
+#if OPENSSL_VERSION_NUMBER < 0x10101000L || defined(LIBRESSL_VERSION_NUMBER)
+static EVP_PKEY_METHOD *orig_method_rsa = NULL;
+#else
+static const EVP_PKEY_METHOD *orig_method_rsa = NULL;
+#endif /* OPENSSL_VERSION_NUMBER < 0x10101000L || defined(LIBRESSL_VERSION_NUMBER) */
+
+static int (*orig_pkey_rsa_sign_init) (EVP_PKEY_CTX *ctx);
+static int (*orig_pkey_rsa_sign) (EVP_PKEY_CTX *ctx,
+	unsigned char *sig, size_t *siglen,
+	const unsigned char *tbs, size_t tbslen);
+static int (*orig_pkey_rsa_decrypt_init) (EVP_PKEY_CTX *ctx);
+static int (*orig_pkey_rsa_decrypt) (EVP_PKEY_CTX *ctx,
+	unsigned char *out, size_t *outlen,
+	const unsigned char *in, size_t inlen);
+#endif /* OPENSSL_VERSION_NUMBER < 0x40000000L */
+
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+# ifndef OPENSSL_NO_EC
 /* DER OIDs */
 static const unsigned char OID_ED25519[] = { 0x06, 0x03, 0x2B, 0x65, 0x70 };
 static const unsigned char OID_ED448[]   = { 0x06, 0x03, 0x2B, 0x65, 0x71 };
@@ -41,8 +62,49 @@ static const unsigned char STR_ED448[] = {
     0x13, 0x0A, /* tag + length */
     'e','d','w','a','r','d','s','4','4','8'
 };
-# endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
-#endif /* OPENSSL_NO_EC */
+# endif /* OPENSSL_NO_EC */
+#endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+
+#if OPENSSL_VERSION_NUMBER < 0x100020d0L || defined(LIBRESSL_VERSION_NUMBER)
+struct evp_pkey_method_st {
+    int pkey_id;
+    int flags;
+    int (*init) (EVP_PKEY_CTX *ctx);
+    int (*copy) (EVP_PKEY_CTX *dst, EVP_PKEY_CTX *src);
+    void (*cleanup) (EVP_PKEY_CTX *ctx);
+    int (*paramgen_init) (EVP_PKEY_CTX *ctx);
+    int (*paramgen) (EVP_PKEY_CTX *ctx, EVP_PKEY *pkey);
+    int (*keygen_init) (EVP_PKEY_CTX *ctx);
+    int (*keygen) (EVP_PKEY_CTX *ctx, EVP_PKEY *pkey);
+    int (*sign_init) (EVP_PKEY_CTX *ctx);
+    int (*sign) (EVP_PKEY_CTX *ctx, unsigned char *sig, size_t *siglen,
+                 const unsigned char *tbs, size_t tbslen);
+    int (*verify_init) (EVP_PKEY_CTX *ctx);
+    int (*verify) (EVP_PKEY_CTX *ctx,
+                   const unsigned char *sig, size_t siglen,
+                   const unsigned char *tbs, size_t tbslen);
+    int (*verify_recover_init) (EVP_PKEY_CTX *ctx);
+    int (*verify_recover) (EVP_PKEY_CTX *ctx,
+                           unsigned char *rout, size_t *routlen,
+                           const unsigned char *sig, size_t siglen);
+    int (*signctx_init) (EVP_PKEY_CTX *ctx, EVP_MD_CTX *mctx);
+    int (*signctx) (EVP_PKEY_CTX *ctx, unsigned char *sig, size_t *siglen,
+                    EVP_MD_CTX *mctx);
+    int (*verifyctx_init) (EVP_PKEY_CTX *ctx, EVP_MD_CTX *mctx);
+    int (*verifyctx) (EVP_PKEY_CTX *ctx, const unsigned char *sig, int siglen,
+                      EVP_MD_CTX *mctx);
+    int (*encrypt_init) (EVP_PKEY_CTX *ctx);
+    int (*encrypt) (EVP_PKEY_CTX *ctx, unsigned char *out, size_t *outlen,
+                    const unsigned char *in, size_t inlen);
+    int (*decrypt_init) (EVP_PKEY_CTX *ctx);
+    int (*decrypt) (EVP_PKEY_CTX *ctx, unsigned char *out, size_t *outlen,
+                    const unsigned char *in, size_t inlen);
+    int (*derive_init) (EVP_PKEY_CTX *ctx);
+    int (*derive) (EVP_PKEY_CTX *ctx, unsigned char *key, size_t *keylen);
+    int (*ctrl) (EVP_PKEY_CTX *ctx, int type, int p1, void *p2);
+    int (*ctrl_str) (EVP_PKEY_CTX *ctx, const char *type, const char *value);
+}; /* EVP_PKEY_METHOD */
+#endif
 
 static int pkcs11_find_keys(PKCS11_SLOT_private *, CK_SESSION_HANDLE, unsigned int,
 	PKCS11_TEMPLATE *);
@@ -983,4 +1045,442 @@ void pkcs11_destroy_keys(PKCS11_SLOT_private *slot, unsigned int type)
 	keys->num = 0;
 }
 
+#if OPENSSL_VERSION_NUMBER < 0x40000000L
+# if OPENSSL_VERSION_NUMBER >= 0x30000000L
+void pkcs11_set_ex_data_pkey(EVP_PKEY *pkey, PKCS11_OBJECT_private *key)
+{
+	EVP_PKEY_set_ex_data(pkey, pkey_ex_index, key);
+}
+
+PKCS11_OBJECT_private *pkcs11_get_ex_data_pkey(const EVP_PKEY *pkey)
+{
+	return EVP_PKEY_get_ex_data(pkey, pkey_ex_index);
+}
+
+void alloc_pkey_ex_index(void)
+{
+	if (pkey_ex_index == 0) {
+		while (pkey_ex_index == 0) /* Workaround for OpenSSL RT3710 */
+			pkey_ex_index = EVP_PKEY_get_ex_new_index(0, "libp11 EVP_PKEY",
+				NULL, NULL, NULL);
+		if (pkey_ex_index < 0)
+			pkey_ex_index = 0; /* Fallback to app_data */
+	}
+}
+
+void free_pkey_ex_index(void)
+{
+	if (pkey_ex_index > 0) {
+		CRYPTO_free_ex_index(CRYPTO_EX_INDEX_EVP_PKEY, pkey_ex_index);
+		pkey_ex_index = 0;
+	}
+}
+# endif /* OPENSSL_VERSION_NUMBER >= 0x30000000L */
+
+static CK_MECHANISM_TYPE pkcs11_md2ckm(const EVP_MD *md)
+{
+	switch (EVP_MD_type(md)) {
+	case NID_sha1:
+		return CKM_SHA_1;
+	case NID_sha224:
+		return CKM_SHA224;
+	case NID_sha256:
+		return CKM_SHA256;
+	case NID_sha512:
+		return CKM_SHA512;
+	case NID_sha384:
+		return CKM_SHA384;
+#if OPENSSL_VERSION_NUMBER >= 0x10101000L && !defined(LIBRESSL_VERSION_NUMBER)
+	case NID_sha3_224:
+		return CKM_SHA3_224;
+	case NID_sha3_256:
+		return CKM_SHA3_256;
+	case NID_sha3_384:
+		return CKM_SHA3_384;
+	case NID_sha3_512:
+		return CKM_SHA3_512;
+#endif
+	default:
+		return 0;
+	}
+}
+
+static CK_RSA_PKCS_MGF_TYPE pkcs11_md2ckg(const EVP_MD *md)
+{
+	switch (EVP_MD_type(md)) {
+	case NID_sha1:
+		return CKG_MGF1_SHA1;
+	case NID_sha224:
+		return CKG_MGF1_SHA224;
+	case NID_sha256:
+		return CKG_MGF1_SHA256;
+	case NID_sha512:
+		return CKG_MGF1_SHA512;
+	case NID_sha384:
+		return CKG_MGF1_SHA384;
+#if OPENSSL_VERSION_NUMBER >= 0x10101000L && !defined(LIBRESSL_VERSION_NUMBER)
+	case NID_sha3_224:
+		return CKG_MGF1_SHA3_224;
+	case NID_sha3_256:
+		return CKG_MGF1_SHA3_256;
+	case NID_sha3_384:
+		return CKG_MGF1_SHA3_384;
+	case NID_sha3_512:
+		return CKG_MGF1_SHA3_512;
+#endif
+	default:
+		return 0;
+	}
+}
+
+static int pkcs11_params_pss(CK_RSA_PKCS_PSS_PARAMS *pss,
+		EVP_PKEY_CTX *ctx, PKCS11_CTX_private *pctx)
+{
+	const EVP_MD *sig_md, *mgf1_md;
+	EVP_PKEY *evp_pkey;
+	int salt_len;
+
+	/* retrieve PSS parameters */
+	if (EVP_PKEY_CTX_get_signature_md(ctx, &sig_md) <= 0)
+		return -1;
+	if (EVP_PKEY_CTX_get_rsa_mgf1_md(ctx, &mgf1_md) <= 0)
+		return -1;
+	if (!EVP_PKEY_CTX_get_rsa_pss_saltlen(ctx, &salt_len))
+		return -1;
+	switch (salt_len) {
+	case -1:
+		salt_len = EVP_MD_size(sig_md);
+		break;
+	case -2:
+		if (!ctx)
+			return -1;
+		evp_pkey = EVP_PKEY_CTX_get0_pkey(ctx);
+		if (!evp_pkey)
+			return -1;
+		salt_len = EVP_PKEY_size(evp_pkey) - EVP_MD_size(sig_md) - 2;
+		if (((EVP_PKEY_bits(evp_pkey) - 1) & 0x7) == 0)
+			salt_len--;
+		if (salt_len < 0) /* integer underflow detected */
+			return -1;
+	}
+	pkcs11_log(pctx, LOG_DEBUG, "salt_len=%d sig_md=%s mdf1_md=%s\n",
+		salt_len, EVP_MD_name(sig_md), EVP_MD_name(mgf1_md));
+
+	/* fill the CK_RSA_PKCS_PSS_PARAMS structure */
+	memset(pss, 0, sizeof(CK_RSA_PKCS_PSS_PARAMS));
+	pss->hashAlg = pkcs11_md2ckm(sig_md);
+	pss->mgf = pkcs11_md2ckg(mgf1_md);
+	if (!pss->hashAlg || !pss->mgf)
+		return -1;
+	pss->sLen = salt_len;
+	return 0;
+}
+
+static int pkcs11_params_oaep(CK_RSA_PKCS_OAEP_PARAMS *oaep,
+		EVP_PKEY_CTX *ctx, PKCS11_CTX_private *pctx)
+{
+	const EVP_MD *oaep_md, *mgf1_md;
+
+	/* retrieve OAEP parameters */
+	if (EVP_PKEY_CTX_get_rsa_oaep_md(ctx, &oaep_md) <= 0)
+		return -1;
+	if (EVP_PKEY_CTX_get_rsa_mgf1_md(ctx, &mgf1_md) <= 0)
+		return -1;
+
+	pkcs11_log(pctx, LOG_DEBUG, "oaep_md=%s mdf1_md=%s\n",
+		EVP_MD_name(oaep_md), EVP_MD_name(mgf1_md));
+
+	/* fill the CK_RSA_PKCS_OAEP_PARAMS structure */
+	memset(oaep, 0, sizeof(CK_RSA_PKCS_OAEP_PARAMS));
+	oaep->hashAlg = pkcs11_md2ckm(oaep_md);
+	oaep->mgf = pkcs11_md2ckg(mgf1_md);
+	if (!oaep->hashAlg || !oaep->mgf)
+		return -1;
+	/* we do not support the OAEP "label" parameter yet... */
+	oaep->source = CKZ_DATA_SPECIFIED;
+	oaep->pSourceData = NULL; /* empty parameter (label) */
+	oaep->ulSourceDataLen = 0;
+	return 0;
+}
+
+/* Attempt to sign using the PKCS#11-backed RSA implementation */
+static int pkcs11_try_pkey_rsa_sign(EVP_PKEY_CTX *evp_pkey_ctx,
+		unsigned char *sig, size_t *siglen,
+		const unsigned char *tbs, size_t tbslen)
+{
+	EVP_PKEY *pkey;
+	RSA *rsa;
+	int rv = 0, padding;
+	CK_ULONG size = (CK_ULONG)*siglen;
+	PKCS11_OBJECT_private *key;
+	PKCS11_SLOT_private *slot;
+	PKCS11_CTX_private *ctx;
+	const EVP_MD *sig_md;
+	CK_SESSION_HANDLE session;
+	CK_MECHANISM mechanism;
+	CK_RSA_PKCS_PSS_PARAMS pss_params;
+
+	/* RSA method has EVP_PKEY_FLAG_AUTOARGLEN set. OpenSSL core will handle
+	 * the size inquiry internally. */
+	if (!sig)
+		return -1;
+	if (!evp_pkey_ctx)
+		return -1;
+
+	pkey = EVP_PKEY_CTX_get0_pkey(evp_pkey_ctx);
+	if (!pkey)
+		return -1;
+	rsa = (RSA *)EVP_PKEY_get0_RSA(pkey);
+	if (!rsa)
+		return -1;
+
+	key = pkcs11_get_ex_data_rsa(rsa);
+	if (check_object_fork(key) < 0)
+		return -1;
+
+	slot = key->slot;
+	ctx = slot->ctx;
+	if (!ctx)
+		return -1;
+#ifdef DEBUG
+	pkcs11_log(ctx, LOG_DEBUG, "%s:%d pkcs11_try_pkey_rsa_sign() "
+		"sig=%p *siglen=%lu tbs=%p tbslen=%lu\n",
+		__FILE__, __LINE__, sig, *siglen, tbs, tbslen);
+#endif
+	if (EVP_PKEY_CTX_get_signature_md(evp_pkey_ctx, &sig_md) <= 0)
+		return -1;
+	if (tbslen != (size_t)EVP_MD_size(sig_md))
+		return -1;
+
+	memset(&mechanism, 0, sizeof mechanism);
+	EVP_PKEY_CTX_get_rsa_padding(evp_pkey_ctx, &padding);
+	switch (padding) {
+	case RSA_PKCS1_PSS_PADDING:
+		pkcs11_log(ctx, LOG_DEBUG, "padding=RSA_PKCS1_PSS_PADDING\n");
+		if (pkcs11_params_pss(&pss_params, evp_pkey_ctx, ctx) < 0)
+			return -1;
+		mechanism.mechanism = CKM_RSA_PKCS_PSS;
+		mechanism.pParameter = &pss_params;
+		mechanism.ulParameterLen = sizeof pss_params;
+		break;
+	default:
+		return -1;
+	} /* end switch(padding) */
+
+	if (pkcs11_get_session(slot, 0, &session))
+		return -1;
+
+	rv = CRYPTOKI_call(ctx,
+		C_SignInit(session, &mechanism, key->object));
+	if (rv != CKR_OK) {
+		pkcs11_log(ctx, LOG_DEBUG, "%s:%d C_SignInit rv=%d\n",
+			__FILE__, __LINE__, rv);
+	} else if (key->always_authenticate == CK_TRUE)
+		rv = pkcs11_authenticate(key, session);
+	if (rv == CKR_OK) {
+		rv = CRYPTOKI_call(ctx,
+			C_Sign(session, (CK_BYTE_PTR)tbs, (CK_ULONG)tbslen, sig, &size));
+		if (rv != CKR_OK) {
+			pkcs11_log(ctx, LOG_DEBUG, "%s:%d C_Sign rv=%d\n",
+				__FILE__, __LINE__, rv);
+		}
+	}
+	pkcs11_put_session(slot, session);
+
+	if (rv != CKR_OK)
+		return -1;
+	*siglen = size;
+	return 1;
+}
+
+/* Attempt to decrypt using the PKCS#11-backed RSA implementation */
+static int pkcs11_try_pkey_rsa_decrypt(EVP_PKEY_CTX *evp_pkey_ctx,
+		unsigned char *out, size_t *outlen,
+		const unsigned char *in, size_t inlen)
+{
+	EVP_PKEY *pkey;
+	RSA *rsa;
+	int rv = 0, padding;
+	CK_ULONG size = (CK_ULONG)*outlen;
+	PKCS11_OBJECT_private *key;
+	PKCS11_SLOT_private *slot;
+	PKCS11_CTX_private *ctx;
+	CK_SESSION_HANDLE session;
+	CK_MECHANISM mechanism;
+	CK_RSA_PKCS_OAEP_PARAMS oaep_params;
+
+	/* RSA method has EVP_PKEY_FLAG_AUTOARGLEN set. OpenSSL core will handle
+	 * the size inquiry internally. */
+	if (!out)
+		return -1;
+	if (!evp_pkey_ctx)
+		return -1;
+
+	pkey = EVP_PKEY_CTX_get0_pkey(evp_pkey_ctx);
+	if (!pkey)
+		return -1;
+	rsa = (RSA *)EVP_PKEY_get0_RSA(pkey);
+	if (!rsa)
+		return -1;
+
+	key = pkcs11_get_ex_data_rsa(rsa);
+	if (check_object_fork(key) < 0)
+		return -1;
+
+	slot = key->slot;
+	ctx = slot->ctx;
+	if (!ctx)
+		return -1;
+#ifdef DEBUG
+	pkcs11_log(ctx, LOG_DEBUG, "%s:%d pkcs11_try_pkey_rsa_decrypt() "
+		"out=%p *outlen=%lu in=%p inlen=%lu\n",
+		__FILE__, __LINE__, out, *outlen, in, inlen);
+#endif
+	memset(&mechanism, 0, sizeof mechanism);
+	EVP_PKEY_CTX_get_rsa_padding(evp_pkey_ctx, &padding);
+	switch (padding) {
+	case RSA_PKCS1_OAEP_PADDING:
+		pkcs11_log(ctx, LOG_DEBUG, "padding=RSA_PKCS1_OAEP_PADDING\n");
+		if (pkcs11_params_oaep(&oaep_params, evp_pkey_ctx, ctx) < 0)
+			return -1;
+		mechanism.mechanism = CKM_RSA_PKCS_OAEP;
+		mechanism.pParameter = &oaep_params;
+		mechanism.ulParameterLen = sizeof oaep_params;
+		break;
+	case RSA_PKCS1_PADDING:
+		pkcs11_log(ctx, LOG_DEBUG, "padding=RSA_PKCS1_PADDING\n");
+		mechanism.mechanism = CKM_RSA_PKCS;
+		mechanism.pParameter = NULL;
+		mechanism.ulParameterLen = 0;
+		break;
+	default:
+		pkcs11_log(ctx, LOG_DEBUG, "%s:%d unsupported padding: %d\n",
+			__FILE__, __LINE__, padding);
+		return -1;
+	} /* end switch(padding) */
+
+	if (pkcs11_get_session(slot, 0, &session))
+		return -1;
+
+	rv = CRYPTOKI_call(ctx,
+		C_DecryptInit(session, &mechanism, key->object));
+	if (rv != CKR_OK) {
+		pkcs11_log(ctx, LOG_DEBUG, "%s:%d C_DecryptInit rv=%d\n",
+			__FILE__, __LINE__, rv);
+	} else if (key->always_authenticate == CK_TRUE)
+		rv = pkcs11_authenticate(key, session);
+	if (rv == CKR_OK) {
+		rv = CRYPTOKI_call(ctx,
+			C_Decrypt(session, (CK_BYTE_PTR)in, (CK_ULONG)inlen, out, &size));
+		if (rv != CKR_OK) {
+			pkcs11_log(ctx, LOG_DEBUG, "%s:%d C_Decrypt rv=%d\n",
+				__FILE__, __LINE__, rv);
+		}
+	}
+	pkcs11_put_session(slot, session);
+
+	if (rv != CKR_OK)
+		return -1;
+	*outlen = size;
+	return 1;
+}
+
+static int pkcs11_pkey_rsa_sign(EVP_PKEY_CTX *evp_pkey_ctx,
+		unsigned char *sig, size_t *siglen,
+		const unsigned char *tbs, size_t tbslen)
+{
+	int ret;
+
+	ret = pkcs11_try_pkey_rsa_sign(evp_pkey_ctx, sig, siglen, tbs, tbslen);
+	if (ret < 0)
+		ret = (*orig_pkey_rsa_sign)(evp_pkey_ctx, sig, siglen, tbs, tbslen);
+	return ret;
+}
+
+static int pkcs11_pkey_rsa_decrypt(EVP_PKEY_CTX *evp_pkey_ctx,
+		unsigned char *out, size_t *outlen,
+		const unsigned char *in, size_t inlen)
+{
+	int ret;
+
+	ret = pkcs11_try_pkey_rsa_decrypt(evp_pkey_ctx, out, outlen, in, inlen);
+	if (ret < 0)
+		ret = (*orig_pkey_rsa_decrypt)(evp_pkey_ctx, out, outlen, in, inlen);
+	return ret;
+}
+
+#if OPENSSL_VERSION_NUMBER < 0x100020d0L || defined(LIBRESSL_VERSION_NUMBER)
+void EVP_PKEY_meth_get_sign(EVP_PKEY_METHOD *pmeth,
+		int (**psign_init) (EVP_PKEY_CTX *ctx),
+		int (**psign) (EVP_PKEY_CTX *ctx,
+			unsigned char *sig, size_t *siglen,
+			const unsigned char *tbs, size_t tbslen))
+{
+	if (psign_init)
+		*psign_init = pmeth->sign_init;
+	if (psign)
+		*psign = pmeth->sign;
+}
+
+static void EVP_PKEY_meth_get_decrypt(EVP_PKEY_METHOD *pmeth,
+		int (**pdecrypt_init) (EVP_PKEY_CTX *ctx),
+		int (**pdecrypt) (EVP_PKEY_CTX *ctx,
+			unsigned char *out,
+			size_t *outlen,
+			const unsigned char *in,
+			size_t inlen))
+{
+	if (pdecrypt_init)
+		*pdecrypt_init = pmeth->decrypt_init;
+	if (pdecrypt)
+		*pdecrypt = pmeth->decrypt;
+}
+#endif
+
+EVP_PKEY_METHOD *pkcs11_pkey_method_rsa(void)
+{
+	EVP_PKEY_METHOD *new_meth_rsa = NULL;
+	int orig_id;
+
+	/* Cache the original EVP_PKEY_RSA method (once) */
+	if (!orig_method_rsa)
+#if OPENSSL_VERSION_NUMBER < 0x10101000L || defined(LIBRESSL_VERSION_NUMBER)
+		orig_method_rsa = (EVP_PKEY_METHOD *)EVP_PKEY_meth_find(EVP_PKEY_RSA);
+#else
+		orig_method_rsa = EVP_PKEY_meth_find(EVP_PKEY_RSA);
+#endif /* OPENSSL_VERSION_NUMBER < 0x10101000L || defined(LIBRESSL_VERSION_NUMBER) */
+
+	if (!orig_method_rsa)
+		return NULL;
+
+	EVP_PKEY_meth_get0_info(&orig_id, NULL, orig_method_rsa);
+	if (orig_id != EVP_PKEY_RSA)
+		return NULL;
+
+	EVP_PKEY_meth_get_sign(orig_method_rsa,
+		&orig_pkey_rsa_sign_init, &orig_pkey_rsa_sign);
+	if (!orig_pkey_rsa_sign)
+		return NULL;
+
+	EVP_PKEY_meth_get_decrypt(orig_method_rsa,
+		&orig_pkey_rsa_decrypt_init, &orig_pkey_rsa_decrypt);
+	if (!orig_pkey_rsa_decrypt)
+		return NULL;
+
+	new_meth_rsa = EVP_PKEY_meth_new(EVP_PKEY_RSA, EVP_PKEY_FLAG_AUTOARGLEN);
+	if (!new_meth_rsa)
+		return NULL;
+
+	/* Duplicate the original method */
+	EVP_PKEY_meth_copy(new_meth_rsa, orig_method_rsa);
+
+	EVP_PKEY_meth_set_sign(new_meth_rsa,
+		orig_pkey_rsa_sign_init, pkcs11_pkey_rsa_sign);
+	EVP_PKEY_meth_set_decrypt(new_meth_rsa,
+		orig_pkey_rsa_decrypt_init, pkcs11_pkey_rsa_decrypt);
+
+	return new_meth_rsa;
+}
+#endif /* OPENSSL_VERSION_NUMBER < 0x40000000L */
+
 /* vim: set noexpandtab: */
